/*Rexx: install cbenable for 2 sockets*/

Call rxfuncadd sysloadfuncs, rexxutil, sysloadfuncs
Call sysloadfuncs
Call SysCls

/*******************************************************************/
/** change following lines and maybe email updates to the author ***/
/*******************************************************************/

/* edit this field if you have an socket services driver that is not in this table */
socketservices_count = 6
socketservices_name.1 = 'IBM2SS14.SYS'          /* IBM driver (thinkpads); ENE Technologies patch */
socketservices_name.2 = 'SS2TICB.SYS'           /* modified IBM2SS14.SYS */
socketservices_name.3 = 'TI1130CB.SYS'          /* TI Chips (Dell) */
socketservices_name.4 = 'TI1225CB.SYS'          /* TI Chips (Dell) */
socketservices_name.5 = 'SSPCIC.SYS'            /* ApSoft - for example Toshiba, TI, Cirrus, O2 Micro, DataBook */
socketservices_name.6 = 'TPIC95SS.SYS'          /* Toshiba */

/* cleanup them from config.sys before installing them again */
uninstall_driver_count = 3
uninstall_driver_name.1 = 'CBCIS.SYS'
uninstall_driver_name.2 = 'OEMHLP2.SYS'
uninstall_driver_name.3 = 'CBENABLE.SYS'

/* number of cbenable instances: 2, and the driver lines
 * short parameter summary:
 * BaseDev=cbenable.sys Bus Device Function Socketnumber (PCIIRQ|NOIRQ) [options...]
 *  <Bus>               = 0..255, use * to automatically find the CardBus device
 *  <Device>            = 0..31, use * to automatically find the CardBus device
 *  <Function>          = 0..7  usualy same as Socketnumber
 *  <Socketnumber>      = 0..7  usualy same as Function, 0=first socket, 1=second socket, and so on
 *  Options:
 *    /Patch_Ti1225_Burst:*
 *    /Patch_Ti1225_IRQMode:*
 *    /FunctionID:*        see documentation
 *    /NoBeep
 */
install_cbenable_count = 2
install_cbenable_line.1 = 'BaseDev=cbenable.sys * * 0 0 PCIIRQ /Patch_Ti1225_Burst:* /Patch_Ti1225_IRQMode:* /FunctionID:*'
install_cbenable_line.2 = 'BaseDev=cbenable.sys * * 1 1 PCIIRQ /Patch_Ti1225_Burst:* /Patch_Ti1225_IRQMode:* /FunctionID:*'
install_cbenable_line.3 = 'BaseDev=cbenable.sys * * 2 2 PCIIRQ /Patch_Ti1225_Burst:* /Patch_Ti1225_IRQMode:* /FunctionID:*'
install_cbenable_line.4 = 'BaseDev=cbenable.sys * * 3 3 PCIIRQ /Patch_Ti1225_Burst:* /Patch_Ti1225_IRQMode:* /FunctionID:*'

/*******************************************************************/

false = 0
true = 1

Say 'This program will install the CardBus Enabler driver package for two sockets.'
Say 'If you have more/less or use point enablers, please change the config.sys yourself.'
Say ''
Say 'Requirements are:'
Say ' * installed PCIConf package'
Say '   always use the newest version..'
Say ' * CardBus hardware'
Say '   should be present in all current/previous PCI based notebooks'
Say ' * installed PCMCIA support'
Say '   either latest Version of 5.00 or 8.00 series'
Say ' * installed CardBus chip socket driver'
Say '   examples: IBM2SS14.SYS/TI1225CB.SYS/SS2TICB.SYS/SSPCIC.SYS(ApSoft)'

Parse Arg cmdl
if Strip(Translate(cmdl))<>'/I' then
  do
    Say 'press q to quit, other key to continue'
    reply = SysGetKey('NoEcho')
    if Translate(reply)='Q' then Return 0
  end

config_sys_name = SysBootDrive()'\config.sys'
backup_sys_name = SysBootDrive()'\config.bak'
config_cmd_name = SysBootDrive()'\config.cmd'
Say 'Reading' config_sys_name '...'

config_sys_count = 0
do while Lines(config_sys_name)
  config_sys_count = config_sys_count + 1
  config_sys_line.config_sys_count = LineIn(config_sys_name)
end
Call Stream config_sys_name, 'C', 'Close'

if Pos('* OS2CFG *',config_sys_line.1) > 0 then
  os2csm_prefix = '^PCMCIA:FALSE:REM ^'
else
  os2csm_prefix = ''

backup_sys_count = config_sys_count
backup_sys_line. = config_sys_line.

/* remove old lines */
do i_uninstall=1 to uninstall_driver_count
  do while true
    uninstall_ln = SearchLine('BASEDEV', '=', uninstall_driver_name.i_uninstall)
    if uninstall_ln == 0 then Leave
    Call RemoveLine uninstall_ln
  end
end

pcmcia_sys_ln = SearchLine('BASEDEV', '=', 'PCMCIA.SYS')
if pcmcia_sys_ln == 0 then
  call error_halt 'Could not find BASEDEV=PCMCIA.SYS line.'

/* CardBus 5.00 needs /R:500 parameter to offer functions. */
/* 8.00 does not need it, but ignores /R:500 */
pcmcia_sys_args = Translate(SearchLineArgs('BASEDEV', '=', 'PCMCIA.SYS'))
if Pos('/R:',pcmcia_sys_args) == 0 then
  config_sys_line.pcmcia_sys_ln = config_sys_line.pcmcia_sys_ln||' /R:500'

/* patch pcmcia.sys driver, must be loaded before any other driver uses pcmcia */
Call InsertLine pcmcia_sys_ln+1, os2csm_prefix||'BaseDev=cbcis.sys'

found_socketservices = 0

/* find last socket services line */
do socketservices_i=1 to socketservices_count
  found_socketservices1 = SearchLine('BASEDEV', '=', socketservices_name.socketservices_i)
  if found_socketservices < found_socketservices1 then
    found_socketservices = found_socketservices1
end

if found_socketservices == 0 then
  call error_halt 'Could not find a known socket services driver line. Maybe i need an update.'

/* the enabler lines, one for each socket */
do i_cbe=1 to install_cbenable_count
  Call InsertLine found_socketservices+1+i_cbe, os2csm_prefix||install_cbenable_line.i_cbe
end

/* OEMHLP2.SYS may be loaded earlier, we use it in cbenable, so place it before.
 * other drivers useing the OEMHLP interface should also use the replacement,
 * so we simply place it as the first line */
Call InsertLine 1, os2csm_prefix||'BaseDev=oemhlp2.sys'

/* write back changes to config.sys */
Say 'Writing' backup_sys_name '...'
'@if exist' backup_sys_name 'del' backup_sys_name
do i=1 to backup_sys_count
  Call LineOut backup_sys_name, backup_sys_line.i
end
Call Stream backup_sys_name, 'C', 'Close'

Say 'Writing' config_sys_name '...'
'@if exist' config_sys_name 'del' config_sys_name
do i=1 to config_sys_count
  Call LineOut config_sys_name, config_sys_line.i
end
Call Stream config_sys_name, 'C', 'Close'

Say 'Copying drivers...'
'@copy cbcis.s??' SysBootDrive()'\OS2\BOOT\'
'@copy oemhlp2.s??' SysBootDrive()'\OS2\BOOT\'
'@copy cbenable.s??' SysBootDrive()'\OS2\BOOT\'

if os2csm_prefix <> '' then
  if Stream(config_cmd_name, 'C', 'Query Exist') <> '' then
    do
      Say 'Updating OS2CSM markup ...'
      '@cmd /c' config_cmd_name '/i'
    end

Say 'Done. Please reboot to load the drivers.'

Return 0

/*******************************************************************/

error_halt:
  Parse Arg message
  Say message
  Say 'press a key to quit'
  reply = SysGetKey('NoEcho')
  Exit 1

/*******************************************************************/

SearchLine:
  Parse Arg s1,s2,s3
  SearchLine_Result = 0

  do l_SearchLine=1 to config_sys_count

    SearchLine_line = config_sys_line.l_SearchLine

    SearchLine_line = Translate(SearchLine_line)
    SearchLine_line = Strip(SearchLine_line)

    /* allow simple OS2CSM constructs */
    if SubStr(SearchLine_line, 1, 3) <> 'REM' then
      if Pos(s1||s2||s3, SearchLine_line) <> 0 then
        do
          SearchLine_Result = l_SearchLine
          Leave l_SearchLine
        end

    if SubStr(SearchLine_line,1,Length(s1)) <> s1 then Iterate
    SearchLine_line = Strip(SubStr(SearchLine_line,Length(s1)+1))

    if SubStr(SearchLine_line,1,Length(s2)) <> s2 then Iterate
    SearchLine_line = Strip(SubStr(SearchLine_line,Length(s2)+1))

    if SubStr(SearchLine_line,1,Length(s3)) <> s3 then Iterate

    SearchLine_Result = l_SearchLine
    Leave l_SearchLine

  end
  Return SearchLine_Result

/*******************************************************************/

SearchLineArgs:
  Parse Arg s1,s2,s3
  SearchLineArgs_Result = ''

  do l_SearchLineArgs=1 to config_sys_count

    SearchLineArgs_line = config_sys_line.l_SearchLineArgs

    SearchLineArgs_line = Translate(SearchLineArgs_line)
    SearchLineArgs_line = Strip(SearchLineArgs_line)

    /* allow simple OS2CSM constructs */
    if SubStr(SearchLineArgs_line, 1, 3) <> 'REM' then
      if Pos(s1||s2||s3, SearchLineArgs_line) <> 0 then
        do
          SearchLineArgs_Result = SubStr(SearchLineArgs_line, Pos(s1||s2||s3, SearchLineArgs_line)+Length(s1||s2||s3))
          Leave l_SearchLineArgs
        end

    if SubStr(SearchLineArgs_line,1,Length(s1)) <> s1 then Iterate
    SearchLineArgs_line = Strip(SubStr(SearchLineArgs_line,Length(s1)+1))

    if SubStr(SearchLineArgs_line,1,Length(s2)) <> s2 then Iterate
    SearchLineArgs_line = Strip(SubStr(SearchLineArgs_line,Length(s2)+1))

    if SubStr(SearchLineArgs_line,1,Length(s3)) <> s3 then Iterate
    SearchLineArgs_line = Strip(SubStr(SearchLineArgs_line,Length(s3)+1))

    SearchLineArgs_Result = SearchLineArgs_line
    Leave l_SearchLineArgs

  end
  Return SearchLineArgs_Result

/*******************************************************************/

RemoveLine:
  Parse Arg LineNum

  if LineNum > config_sys_count then
    call error_halt 'Invalid line number to delete.'

  do i_RemoveLine=LineNum+1 to config_sys_count
    i_RemoveLine_M1 = i_RemoveLine - 1
    config_sys_line.i_RemoveLine_M1 = config_sys_line.i_RemoveLine
  end

  config_sys_count = config_sys_count - 1

  Return

/*******************************************************************/

InsertLine:
  Parse Arg LineNum, LineValue

  if LineNum > config_sys_count+1 then
    call error_halt 'Invalid line number to insert.'

  do i_InsertLine=config_sys_count to LineNum by -1
    i_InsertLine_P1 = i_InsertLine + 1
    config_sys_line.i_InsertLine_P1 = config_sys_line.i_InsertLine
  end

  config_sys_line.LineNum = LineValue

  config_sys_count = config_sys_count + 1

  Return

/*******************************************************************/
